@@ -31,9 +31,8 @@ struct _KgxCloseDialog {
 
   KgxCloseDialogContext  context;
   GPtrArray             *commands;
-  GObject               *transient_for;
 
-  AdwMessageDialog      *dialog;
+  AdwAlertDialog        *dialog;
   GtkWidget             *list;
 };
 
@@ -44,39 +43,19 @@ enum {
   PROP_0,
   PROP_CONTEXT,
   PROP_COMMANDS,
-  PROP_TRANSIENT_FOR,
   LAST_PROP
 };
 static GParamSpec *pspecs[LAST_PROP] = { NULL, };
 
 
-static void
-lost_parent (gpointer data, GObject *dead)
-{
-  KgxCloseDialog *self = data;
-
-  self->transient_for = NULL;
-  g_object_notify_by_pspec (G_OBJECT (self), pspecs[PROP_TRANSIENT_FOR]);
-}
-
-
 static void
 kgx_close_dialog_dispose (GObject *object)
 {
   KgxCloseDialog *self = KGX_CLOSE_DIALOG (object);
 
   g_clear_pointer (&self->commands, g_ptr_array_unref);
-
-  if (self->dialog) {
-    gtk_window_destroy (GTK_WINDOW (self->dialog));
-  }
   g_clear_weak_pointer (&self->dialog);
 
-  if (self->transient_for) {
-    g_object_weak_unref (self->transient_for, lost_parent, self);
-    self->transient_for = NULL;
-  }
-
   G_OBJECT_CLASS (kgx_close_dialog_parent_class)->dispose (object);
 }
 
@@ -146,15 +125,6 @@ kgx_close_dialog_set_property (GObject      *object,
       self->commands = g_value_dup_boxed (value);
       update_dialog (self);
       break;
-    case PROP_TRANSIENT_FOR:
-      if (self->transient_for) {
-        g_object_weak_unref (self->transient_for, lost_parent, self);
-      }
-      self->transient_for = g_value_get_object (value);
-      if (G_LIKELY (self->transient_for)) {
-        g_object_weak_ref (self->transient_for, lost_parent, self);
-      }
-      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
       break;
@@ -177,9 +147,6 @@ kgx_close_dialog_get_property (GObject    *object,
     case PROP_COMMANDS:
       g_value_set_boxed (value, self->commands);
       break;
-    case PROP_TRANSIENT_FOR:
-      g_value_set_object (value, self->transient_for);
-      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
       break;
@@ -207,11 +174,6 @@ kgx_close_dialog_class_init (KgxCloseDialogClass *klass)
                         G_TYPE_PTR_ARRAY,
                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
-  pspecs[PROP_TRANSIENT_FOR] =
-    g_param_spec_object ("transient-for", NULL, NULL,
-                         GTK_TYPE_WINDOW,
-                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
-
   g_object_class_install_properties (object_class, LAST_PROP, pspecs);
 }
 
@@ -219,7 +181,7 @@ kgx_close_dialog_class_init (KgxCloseDialogClass *klass)
 static void
 kgx_close_dialog_init (KgxCloseDialog *self)
 {
-  AdwMessageDialog *dialog;
+  AdwAlertDialog *dialog;
 
   self->list = g_object_new (GTK_TYPE_LIST_BOX,
                              "can-focus", FALSE,
@@ -232,24 +194,21 @@ kgx_close_dialog_init (KgxCloseDialog *self)
                                   GTK_ACCESSIBLE_PROPERTY_LABEL, _("Process list"),
                                   -1);
 
-  dialog = g_object_new (ADW_TYPE_MESSAGE_DIALOG,
+  dialog = g_object_new (ADW_TYPE_ALERT_DIALOG,
                          "close-response", "cancel",
                          "extra-child", self->list,
                          NULL);
   g_set_weak_pointer (&self->dialog, dialog);
-  g_object_bind_property (self, "transient-for",
-                          self->dialog, "transient-for",
-                          G_BINDING_DEFAULT);
 
-  adw_message_dialog_add_responses (self->dialog,
-                                    /* Translators: This action dismisses the dialogue, leaving the tab/window open */
-                                    "cancel", _("_Cancel"),
-                                    /* Translators: This action accepts the ‘unexpected outcomes’ and closes the tab/window */
-                                    "close", _("C_lose"),
-                                    NULL);
-  adw_message_dialog_set_response_appearance (self->dialog,
-                                              "close",
-                                              ADW_RESPONSE_DESTRUCTIVE);
+  adw_alert_dialog_add_responses (self->dialog,
+                                  /* Translators: This action dismisses the dialogue, leaving the tab/window open */
+                                  "cancel", _("_Cancel"),
+                                  /* Translators: This action accepts the ‘unexpected outcomes’ and closes the tab/window */
+                                  "close", _("C_lose"),
+                                  NULL);
+  adw_alert_dialog_set_response_appearance (self->dialog,
+                                            "close",
+                                            ADW_RESPONSE_DESTRUCTIVE);
 }
 
 
@@ -259,7 +218,7 @@ got_choice (GObject *source, GAsyncResult *res, gpointer user_data)
   g_autoptr (GTask) task = user_data;
   const char *choice;
 
-  choice = adw_message_dialog_choose_finish (ADW_MESSAGE_DIALOG (source), res);
+  choice = adw_alert_dialog_choose_finish (ADW_ALERT_DIALOG (source), res);
 
   if (!g_strcmp0 (choice, "close")) {
     g_task_return_int (task, KGX_CLOSE_ANYWAY);
@@ -270,24 +229,26 @@ got_choice (GObject *source, GAsyncResult *res, gpointer user_data)
 
 
 void
-kgx_close_dialog_run (KgxCloseDialog      *self,
-                      GCancellable        *cancellable,
-                      GAsyncReadyCallback  callback,
-                      gpointer             user_data)
+kgx_close_dialog_run (KgxCloseDialog      *restrict self,
+                      GtkWidget           *restrict parent,
+                      GCancellable        *restrict cancellable,
+                      GAsyncReadyCallback           callback,
+                      gpointer                      user_data)
 {
   g_autoptr (GTask) task = g_task_new (self, cancellable, callback, user_data);
 
-  adw_message_dialog_choose (self->dialog,
-                             cancellable,
-                             got_choice,
-                             g_steal_pointer (&task));
+  adw_alert_dialog_choose (self->dialog,
+                           parent,
+                           cancellable,
+                           got_choice,
+                           g_steal_pointer (&task));
 }
 
 
 KgxCloseDialogResult
-kgx_close_dialog_run_finish (KgxCloseDialog  *self,
-                             GAsyncResult    *res,
-                             GError         **error)
+kgx_close_dialog_run_finish (KgxCloseDialog  *restrict self,
+                             GAsyncResult    *restrict res,
+                             GError         **restrict error)
 {
   g_return_val_if_fail (KGX_IS_CLOSE_DIALOG (self), KGX_CLOSE_CANCELLED);
   g_return_val_if_fail (g_task_is_valid (res, self), KGX_CLOSE_CANCELLED);
